@@ -1,10 +1,10 @@
 from datetime import datetime
+from random import randint
 
-from sqlalchemy import UniqueConstraint, or_
+from sqlalchemy import UniqueConstraint, event
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import validates
 
-
 from investing_algorithm_framework.core.models import db, OrderSide, Order, \
     OrderStatus, OrderType
 from investing_algorithm_framework.core.exceptions import OperationalException
@@ -51,6 +51,9 @@ class Portfolio(db.Model, ModelExtension):
     unallocated = db.Column(db.Float, nullable=False, default=0)
     realized = db.Column(db.Float, nullable=False, default=0)
 
+    total_revenue = db.Column(db.Float, nullable=False, default=0)
+    total_cost = db.Column(db.Float, nullable=False, default=0)
+
     created_at = db.Column(db.DateTime, default=datetime.utcnow())
     updated_at = db.Column(
         db.DateTime,
@@ -86,33 +89,17 @@ class Portfolio(db.Model, ModelExtension):
     def __init__(
             self, trading_symbol, unallocated, identifier, market, **kwargs
     ):
+        self.id = randint(1, 10)
         self.identifier = identifier
         self.trading_symbol = trading_symbol
         self.unallocated = unallocated
-        self.realized = unallocated
+        self.realized = 0
+        self.total_revenue = 0
+        self.total_cost = 0
         self.market = market
+        self.created_at = datetime.utcnow()
         super(Portfolio, self).__init__(**kwargs)
 
-    @hybrid_property
-    def delta(self):
-        from investing_algorithm_framework.core.models import Position
-
-        position_ids = self.positions.with_entities(Position.id)
-
-        orders = Order.query \
-            .filter_by(order_side=OrderSide.BUY.value) \
-            .filter_by(status=OrderStatus.SUCCESS.value) \
-            .filter_by(closing_price=None)\
-            .filter(Order.position_id.in_(position_ids)) \
-            .all()
-
-        delta = 0
-
-        for order in orders:
-            delta += order.delta
-
-        return delta
-
     @hybrid_property
     def allocated(self):
         from investing_algorithm_framework.core.models import Position
@@ -121,8 +108,7 @@ class Portfolio(db.Model, ModelExtension):
 
         orders = Order.query \
             .filter_by(order_side=OrderSide.BUY.value) \
-            .filter(or_(Order.status == OrderStatus.SUCCESS.value, Order.status == OrderStatus.PENDING.value, Order.status == None)) \
-            .filter_by(closing_price=None)\
+            .filter_by(status=OrderStatus.SUCCESS.value)\
             .filter(Order.position_id.in_(position_ids)) \
             .all()
 
@@ -135,16 +121,36 @@ class Portfolio(db.Model, ModelExtension):
 
     @hybrid_property
     def allocated_percentage(self):
+
+        # Prevent zero division
+        if self.allocated == 0 and self.unallocated == 0:
+            return 0
+
         return (self.allocated / (self.allocated + self.unallocated)) * 100
 
     @hybrid_property
     def unallocated_percentage(self):
-        return (self.unallocated / self.realized) * 100
 
-    def add_order(self, order, user_ids=None):
+        # Prevent zero division
+        if self.allocated == 0 and self.unallocated == 0:
+            return 0
+
+        return (self.unallocated / self.unallocated + self.allocated) * 100
+
+    def add_order(self, order):
         self._validate_order(order)
         self._add_order_to_position(order)
 
+        order.update(db, {"status": OrderStatus.TO_BE_SENT.value}, commit=True)
+
+        if OrderSide.BUY.equals(order.order_side):
+            self.unallocated -= order.amount_trading_symbol
+
+            # Create the snapshot at the time the order was created
+            self.snapshot(creation_datetime=order.created_at)
+
+        db.session.commit()
+
     def _validate_order(self, order):
         order_validator = OrderValidatorFactory.of(self.market)
         order_validator.validate(order, self)
@@ -152,11 +158,12 @@ class Portfolio(db.Model, ModelExtension):
     def _add_order_to_position(self, order):
         from investing_algorithm_framework.core.models import Position
 
+        position = Position.query \
+            .filter_by(symbol=order.target_symbol) \
+            .filter_by(portfolio=self) \
+            .first()
+
         if OrderSide.BUY.equals(order.order_side):
-            position = Position.query \
-                .filter_by(symbol=order.target_symbol) \
-                .filter_by(portfolio=self)\
-                .first()
 
             if position is None:
                 position = Position(symbol=order.target_symbol)
@@ -164,10 +171,6 @@ class Portfolio(db.Model, ModelExtension):
                 self.positions.append(position)
                 db.session.commit()
         else:
-            position = Position.query \
-                .filter_by(symbol=order.trading_symbol) \
-                .filter_by(portfolio=self)\
-                .first()
 
             if position is None:
                 raise OperationalException(
@@ -183,7 +186,8 @@ class Portfolio(db.Model, ModelExtension):
         order_type,
         symbol,
         price=None,
-        amount=None,
+        amount_trading_symbol=None,
+        amount_target_symbol=None,
         order_side=OrderSide.BUY.value,
         validate_pair=True,
     ):
@@ -205,64 +209,81 @@ class Portfolio(db.Model, ModelExtension):
 
         if OrderType.MARKET.equals(order_type):
 
-            if OrderSide.BUY.equals(order_side):
+            if OrderSide.SELL.equals(order_side):
                 order = Order(
                     target_symbol=symbol,
                     trading_symbol=self.trading_symbol,
-                    amount_trading_symbol=amount,
-                    price=price,
-                    order_type=order_type,
-                    order_side=OrderSide.BUY.value
+                    amount_target_symbol=amount_target_symbol,
+                    order_type=OrderType.MARKET.value,
+                    order_side=OrderSide.SELL.value,
+                    price=price
                 )
             else:
-                order = Order(
-                    target_symbol=self.trading_symbol,
-                    trading_symbol=symbol,
-                    amount_trading_symbol=amount,
-                    price=price,
-                    order_type=order_type,
-                    order_side=OrderSide.SELL.value
-                )
+                raise OperationalException("Buy market order is not supported")
         else:
-            if OrderSide.BUY.equals(order_side):
-                order = Order(
-                    target_symbol=symbol,
-                    trading_symbol=self.trading_symbol,
-                    amount=amount,
-                    amount_trading_symbol=amount * price,
-                    price=price,
-                    order_type=order_type,
-                    order_side=OrderSide.BUY.value
-                )
-            else:
-                order = Order(
-                    target_symbol=self.trading_symbol,
-                    trading_symbol=symbol,
-                    amount=amount,
-                    amount_trading_symbol=amount * price,
-                    price=price,
-                    order_type=order_type,
-                    order_side=OrderSide.SELL.value
-                )
+            order = Order(
+                target_symbol=symbol,
+                trading_symbol=self.trading_symbol,
+                amount_target_symbol=amount_target_symbol,
+                amount_trading_symbol=amount_trading_symbol,
+                order_type=order_type,
+                order_side=order_side,
+                price=price
+            )
 
         return order
 
+    def withdraw(self, amount, creation_datetime=datetime.utcnow()):
+
+        if amount > self.unallocated:
+            raise OperationalException(
+                "Withdrawal is larger then unallocated size"
+            )
+
+        self.unallocated -= amount
+        db.session.commit()
+        self.snapshot(creation_datetime=creation_datetime, withdrawel=amount)
+
+    def deposit(self, amount, creation_datetime=datetime.utcnow()):
+        self.unallocated += amount
+        db.session.commit()
+        self.snapshot(creation_datetime=creation_datetime, deposit=amount)
+
     def update(self, db, data, commit: bool = False, **kwargs):
         self.updated_at = datetime.utcnow()
         unallocated = data.pop("unallocated", None)
 
         if unallocated is not None:
             self.unallocated += unallocated
-            self.realized += unallocated
+            self.snapshot()
 
         super(Portfolio, self).update(db, data, **kwargs)
 
+    def snapshot(
+        self, withdrawel=0, deposit=0, commit=True, creation_datetime=None
+    ):
+        from investing_algorithm_framework.core.models.snapshots \
+            import PortfolioSnapshot
+
+        PortfolioSnapshot\
+            .from_portfolio(
+                self, creation_datetime, withdrawel=withdrawel, deposit=deposit
+            ).save(db, commit=commit)
+
     def __repr__(self):
         return self.repr(
             id=self.id,
-            trading_currency=self.trading_currency,
-            unallocated=f"{self.unallocated} {self.trading_currency}",
+            trading_symbol=self.trading_symbol,
+            unallocated=f"{self.unallocated} {self.trading_symbol}",
+            realized=f"{self.realized}",
+            total_revenue=f"{self.total_revenue}",
             identifier=self.identifier,
             created_at=self.created_at,
             updated_at=self.updated_at
         )
+
+
+# first snapshot on creation of a portfolio
+@event.listens_for(Portfolio, 'before_insert')
+def snapshot_after_insert(mapper, connection, portfolio):
+    portfolio.snapshot(commit=False)
